(clear-all)

(require-extra "associative-learning")

(define-model SF-player
  (setf *changes* (blank-changes))
  (setf *record* nil)
  (setf *explained*  nil)
  (setf *lastkey* nil)
  (setf *lastaction* nil)
  (setf *kill* nil)
  (setf *vulnerability-kill* nil)
   (setf *vulnerability-reset* nil)

  (sgp :er t :esc t :egs .0001 :ol t :lf .1  :ans .01 :rt -1 :v nil :pct nil :trace-detail low :al-show nil)
  ;; Temporal module's noise
  (sgp :sf-data-hook record-features-fortress)
 
;; Temporal module's noise
  (sgp :TIME-NOISE .005)
  
 ;; Motor setup, timing, and randomization 
  (sgp :randomize-time 3)
  (sgp :dual-execution-stages t)
  (sgp :MOTOR-FEATURE-PREP-TIME 0.020)
  (sgp :MOTOR-INITIATION-TIME .020)
 ;associative learning
  (sgp  :eal .5 :al-prior-p .1 :al-prior-w 1)  
  ;; visual configuration 
  (sgp  :visual-movement-tolerance 2 :do-not-harvest visual)
  (sgp :ga 1.0 :imaginal-activation 0.0  :mas 2 :nsji nil)

  (chunk-type gamestate x y orientation vx vy ship-alive status game predicted-VULNERABILITY change missiles   PRIOR-STATE
               points time-to-outer thrust-angle vulnerability FORTRESS-ANGLE fortress-alive outward prop-to-big speed)   
  (chunk-type goal state predicted-VULNERABILITY reset-bound kill-bound start thrust-time time-thresh
              achieve avoid step other-path type checking-avoid change
              NEXT-SHOT-LENGTH NEXT-THRUST-LENGTH NEXT-SHOT-TIME NEXT-THRUST-TIME)
  (chunk-type operator pre action arg1 result post  fail success type)
  (chunk-type tracker rectangle death  VULNERABILITY-RESET
              hit miss badspeed goodspeed bighexdeath shelldeath smallhexdeath)
  (chunk-type predction predicted type)
  (chunk-type causal cause effect condition1 condition2 type)

(define-chunks yes no speed adjust test-dimension death  prop-to-big w spacebar time-to-outer thrust-angle vulnerability causal
                      suspend vuln-inc big-losses double-w spacefortress vuln-zero points-inc inward ship-death border test-less TEST-GREATER-EQUAL
                      done connected start-playing-again game-over start-playing  operator PREDICTED-VULNERABILITY
                      CHECK-DELAY ANGLE fortress-angle  double-spacebar vulnerability? reset hit time-thresh bighexdeath opposing 
                      delay vuln>10 big-gain big-loss avoid-bad avoiding achieve-good achieving building-test building-test2 building-action finished-instructions achieving-condition
                      prediction kill  shelldeath smallhexdeath thrust-time miss badspeed goodspeed state-changed)

(add-effects)

(sgp :ul t :epl t :egs .05 :iu 9  ) 

(sgp-fct (list :alpha *alpha* :initial-temp *factor* :v *v*))

  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Productions for processing causal instructions
(p avoid-bad  ;avoid bad things first
  =goal>
    isa goal
    step avoid-bad
    avoid =bad
==>
  +retrieval>
     isa causal
     effect =bad
   =goal>
     step avoiding)

(p test-for-danger ;test for a dangerous condition
  =goal>
    isa goal
    step avoiding
  =retrieval>
     condition1 nil
     cause =cause
  ?imaginal>
       state free
==>
  !bind! =oppose (oppose-condition =cause)
  !bind! =test (build-operator =cause "start")
  +imaginal> =test
  +retrieval>
     isa causal
     effect =oppose
   =goal>
     step opposing)

(p build-opposing-action ;build action to avoid dangerous condition
  =goal>
    isa goal
    step opposing
  =retrieval>
     condition1 nil
     cause =action
  =imaginal>
       success =path
       fail =other-path
  ?imaginal>
       state free
==>
  !bind! =test (build-operator =action =path)
  +imaginal> =test
   =goal>
     step achieve-good
     other-path =other-path)

(p achieve-good ;find out what causes a good thing
  =goal>
    isa goal
    step achieve-good
    achieve =good
==>
  +retrieval>
     isa causal
     effect =good
   =goal>
     step achieving)

(p test-for-condition1; build test if ready to perform action
  =goal>
    isa goal
    step achieving
    other-path =path
  =retrieval>
     condition1 =condition
  ?imaginal>
       state free
==>
  !bind! =test (build-operator =condition =path)
  +imaginal> =test
  =retrieval>
   =goal>
     step building-test)

(p test-for-condition2 ;a second test for action
  =goal>
    isa goal
    step building-test
  =retrieval>
     condition2 =condition
  =imaginal>
       success =path
       fail =other-path
  ?imaginal>
       state free
==>
  !bind! =test (build-operator =condition =path)
  +imaginal> =test
  =retrieval>
   =goal>
     step building-test2
     other-path =other-path)

(p build-action ;ready to build action
  =goal>
    isa goal
    step building-test2
  =retrieval>
     condition3 nil
     cause =action
  =imaginal>
       success =path
  ?imaginal>
       state free
==>
  !bind! =test (build-operator =action =path)
  +imaginal> =test
  =retrieval>
   =goal>
     step building-action)

(p build-action2  ;also ready to build action
  =goal>
    isa goal
    step building-test
  =retrieval>
     condition2 nil
     cause =action
  =imaginal>
       success =path
  ?imaginal>
       state free
==>
  !bind! =test (build-operator =action =path)
  +imaginal> =test
   =goal>
     step finished-instructions)

(p achieving-condition1 ; find out what to do if condition1 not satisfied
  =goal>
    isa goal
    step building-action
  =retrieval>
     condition1 =condition
==>
  !bind! =repair (achieve-condition =condition)
  +retrieval>
     isa causal
     effect =repair
   =goal>
     step achieving-condition)

(p build-subcondition ; build test for a subcondition of a condition 
  =goal>
    isa goal
    step achieving-condition
    other-path =path
  =retrieval>
     condition1 =condition
  ?imaginal>
       state free
==>
  !bind! =test (build-operator =condition =path)
  +imaginal> =test
  =retrieval>
   =goal>
     step building-test
     other-path nil)









  
;productions for starting and ending games
(p get-started-discover
     "Make sure game is running"
     ?game>
       game-state connected
     ?goal>
       buffer empty
     ?imaginal>
       state free
     ?game-state>
       state free
     ==>
     +imaginal>
        isa tracker
!bind! =thrust-delay (thrust-time)
!bind! =thrust-length (press-down .15)
!bind! =shot-delay (shoot-time)
!bind! =shot-length (press-down .15)
    +temporal>
       isa time 
     +goal>
         isa goal
         state "experiment"
         next-thrust-time =thrust-delay
         next-shot-time =shot-delay
         next-thrust-length =thrust-length
         next-shot-length =shot-length
     +game-state>
        isa gamestate)

  (p get-started-instructed
     "Make sure game is running"
     ?game>
       game-state connected
     =goal>
       step finished-instructions
     ?imaginal>
       state free
     ?game-state>
       state free
     ==>
     +imaginal>
        isa tracker
    +temporal>
       isa time 
     =goal>
         step nil
         state start-playing 
     +game-state>
        isa gamestate)

 (p start-playing-spacefortress
     "once the ship is visible get going"
     =goal>
       isa goal
       state start-playing  
    =game-state>
        game spacefortress    
     ==>
     +tracker>
         control-slot time-thresh
         good-slot hit
         bad-slot reset
         min 9.0
         max 18.0
         bad-weight -1
     +tracker>
         control-slot time-to-outer 
         good-slot kill
         bad-slot death
         min 1.0
         max 5.0
         bad-weight -1
     +tracker>
         control-slot thrust-time
         good-slot kill
         bad-slot death
         min 0.067
         max 0.167
         bad-weight -1
     =goal>
         state "play"
         start "start"
         predicted-vulnerability 10
    +temporal>
       isa time)

(p get-started-again  ;continues "play" for another game.
     "Make sure game is running"
     ?game>
       game-state connected
    =game-state>
       ship-alive yes 
     =goal>
       state game-over
     ?imaginal>
       state free
==>
     +imaginal>
        isa tracker
    +temporal>
       isa time 
    =game-state>
      status nil
     =goal>
       prior-state nil
       state "play")

 (p detect-trial-end   ;determines game over and requires that information in game-state
    "If a trial is clear things out"
    =goal>
      isa goal
    -  state game-over
    =game-state>
      status game-over
    ?manual>
      processor free
==>
    =goal>
      state game-over
    +manual>
     isa release-all-fingers
    )











;Productions for experimenting, making causal inferences and building operators
(p experiment-w
   =goal>
     next-thrust-time =time
     next-thrust-length =length
   =game-state>
       ship-alive yes
    ?manual>
      state free
!eval! (< =time (/ (get-time) 1000.0))
     ==>
!bind! =thrust-delay (thrust-time)
!bind! =thrust-length (press-down .1)
   +MANUAL>
       CMD DELAYED-PUNCH
       FINGER MIDDLE
       HAND LEFT
       delay =length
   =goal>
         next-thrust-time =thrust-delay
         next-thrust-length =thrust-length)

(p experiment-spacebar
   =goal>
     next-shot-time =time
     next-shot-length =length
   =game-state>
       ship-alive yes
    ?manual>
      state free
!eval! (< =time (/ (get-time) 1000.0))
     ==>
!bind! =shot-delay (shoot-time)
!bind! =shot-length  (press-down .15)
   +MANUAL>
       isa delayed-punch
        hand right
       finger index
       delay =length
   =goal>
         next-shot-time =shot-delay
         next-shot-length =shot-length)

(p rehearse-effect ;retrieves effect primed by possible cause
     =game-state>
         prior-state =state
         change =change
     =goal>
         - prior-state =state    
      -  state "do-step"
     ?retrieval>
           state free
          - buffer full
==>
     =game-state>
         prior-state nil
         change nil
   =goal>
      prior-state =state
   +retrieval>
       predicted =change
!eval! (update-changes =state =change))

(p ignore-cause-effect ;while the effect was retrieve it was too slow to trigger causal inference
  ?imaginal>
      state free
  =goal>
    prior-state =factor
   =retrieval>
        predicted =effect
!eval! (not (retrieval-time-thresh =factor =retrieval))
     ==> 
=goal>
   prior-state nil)

(p attend-cause-effect   ;retrieval was fast enogu to trigger causal inference
 ?imaginal>
      state free
  =goal>
    prior-state =factor
   =retrieval>
        predicted =effect
!eval! (retrieval-time-thresh =factor =retrieval)
     ==> 
!eval! (setf *explained* (cons =effect *explained*))
!eval! (setf *record* (append *record* (list (/ (get-time) 1000.0) =factor =effect)))
=goal>
   prior-state nil
   +imaginal>
        type causal
        cause =factor
        effect =effect)

(p empty-effect  ;after causal inference created in imaginal, return imaginal to its role in tracking
      =imaginal>
          effect =effect
     ?imaginal>
        state free
==>
     +imaginal>
        isa tracker)

(p subgoal-avoids-border1 ;if a ship death or big-loss has occurred after a inward 
   =imaginal>
        isa causal
        cause border
        effect =effect
   =goal>
!eval! (member 'inward *explained*)
!eval! (or (and (equal =effect 'ship-death) (not (member 'big-losses *explained*)))
           (and (equal =effect 'big-losses) (not (member 'ship-death *explained*))))
==>
     !eval! (w-dm)
     +tracker>
         control-slot time-to-outer 
         good-slot kill
         bad-slot death
         min 1.0
         max 5.0
         bad-weight -1
     +tracker>
         control-slot thrust-time
         good-slot kill
         bad-slot death
         min 0.067
         max 0.167
         bad-weight -1
    =goal>
       state "play"
       start "shoot-path"
       NEXT-THRUST-TIME nil
       next-thrust-length nil)


(p subgoal-avoids-border2   ;if a inward causal inference has occurred after a ship death or big-loss
   =imaginal>
        isa causal
        cause w
        effect inward
   =goal>
!eval! (or (member 'ship-death *explained*) (member 'big-losses *explained*))
==>
     !eval! (w-dm)
     +tracker>
         control-slot time-to-outer 
         good-slot kill
         bad-slot death
         min 1.0
         max 5.0
         bad-weight -1
     +tracker>
         control-slot thrust-time
         good-slot kill
         bad-slot death
         min 0.067
         max 0.167
         bad-weight -1
    =goal>
       state "play"
       start "shoot-path"
       NEXT-THRUST-TIME nil
       next-thrust-length nil  )

(p build-shooter1  ;if a vulnerability increment inference has occurred after a reset inference
   =imaginal>
        isa causal 
        cause spacebar    
        effect vuln-inc
   =goal>
!eval! (member 'vuln-zero *explained*)
     ?imaginal>
       state free
==>
    !eval! (space-dm)
    !bind! =start (get-shoot-node)
    =goal>
       state "play"    
       start =start
       NEXT-shot-TIME nil
       next-shot-length nil   
     +tracker>
         control-slot time-thresh
         good-slot hit
         bad-slot reset
         min 5.0
         max 18.0
         bad-weight 0
         good-weight 0
     +imaginal>
        isa tracker)

(p build-shooter2  ;if a  a reset inference after a vulnerability increment inference 
   =imaginal>
        isa causal 
        cause double-spacebar    
        effect vuln-zero
   =goal>
!eval! (member 'vuln-inc *explained*)
     ?imaginal>
       state free
==>
    !eval! (space-dm)
    !bind! =start (get-shoot-node)
    =goal>
       state "play"    
       start =start
       NEXT-shot-TIME nil
       next-shot-length nil   
     +tracker>
         control-slot time-thresh
         good-slot hit
         bad-slot reset
         min 5.0
         max 18.0
         bad-weight 0
         good-weight 0
     +imaginal>
        isa tracker)

(p double-good  ;if a double-spacebar has been inferred to produced a big point increase 
   =imaginal>
        isa causal 
        cause double-spacebar    
        effect points-inc
     ?imaginal>
       state free
   =goal>
==>
    !eval! (vulnerability-dm)
     +tracker>
         control-slot predicted-vulnerability
         good-slot kill
         bad-slot vulnerability-reset
         min 5.0
         max 15.0
         bad-weight -1   
     +tracker>
         control-slot time-thresh
         good-slot hit
         bad-slot reset
         min 5.0
         max 18.0
         bad-weight -1 
     +imaginal>
        isa tracker
    =goal>
       reset-bound 5
       kill-bound 15
       state "play"
       start "start")

(p trigger-vulnerability  ;when the vulnerability range has been completely narrowed.
   =goal>
       reset-bound =val
       kill-bound =val
    -   PREDICTED-VULNERABILITY =val
!eval! (not (member 'vulnerability *explained*))
     ==> 
!eval! (setf *explained* (cons 'vulnerability *explained*))
!eval! (setf *record* (append *record* (list (/ (get-time) 1000.0) =val 'vulnerability)))
     +tracker>
       isa suspend-tracker
       control-slot predicted-vulnerability
   =goal>
      predicted-vulnerability =val)



;productions for play driven by operators
(p do-something  ;starts at top of operator tree
       =goal>
          isa goal
          state "play"
         start =start
      ?retrieval>
           state free
       ?MANUAL-left>
          state FREE
      ?MANUAl-right>
        processor FREE
       ?vocal>
            state free
      ==>
        =goal>
           state "do-step"
           prior-state nil
        +retrieval>
           isa operator
           pre =start)

 (p pause ;pause if there has been an error in retrieving an operator 
         ;tends to happen when a branch in the operator tree has yet to built
       =goal>
          isa goal
          state "do-step"
          start =start
      ?retrieval>
           state error
           state free
       ?MANUAL-left>
          state FREE
      ?MANUAl-right>
        processor FREE
       ?vocal>
            state free
    =game-state>
        game =game 
      ==>
        =goal>
           state "pause"
        +vocal>
           cmd speak
           string "wait")

(p do-something1 ;start again at the top of the operator tree after the pause
       =goal>
          isa goal
          state "pause"
          start =start
       ?vocal>
            state free
    =game-state>
        game =game 
      ==>
        =goal>
           prior-state nil
           state "play"
        +retrieval>
           isa operator
           pre =start)

(p test-less-than-success ;when the value is less than the threshold
   =goal>
      state "do-step"
      =dimension =thresh
    =retrieval>
      isa operator
      action test-less
      arg1 =dimension
      success =newstep
    =game-state>
        =dimension =val     
   !SAFE-EVAL! (< =val =thresh)
     ==>
        =goal>
           prior-state nil
    +retrieval>
        isa operator
        pre =newstep)

(p test-less-than-fail ;when the value is not less than the threshold
   =goal>
      state "do-step"
      =dimension =thresh
    =retrieval>
      isa operator
      action test-less
      arg1 =dimension
      fail =newstep
    =game-state>
        =dimension =val     
   !safe-eval! (>= =val =thresh)
     ==>
    +retrieval>
        isa operator
        pre =newstep)

(p test-greater-than-success ;when the value is greater than the threshold
   =goal>
      state "do-step"
      =dimension =thresh
    =retrieval>
      isa operator
      action test-greater-equal
      arg1 =dimension
      success =newstep
    =game-state>
        =dimension =val     
   !SAFE-EVAL! (>= =val =thresh)
     ==>
        =goal>
           prior-state nil
    +retrieval>
        isa operator
        pre =newstep)

(p test-greater-than-fail ;when the value is not greater than the threshold
   =goal>
      state "do-step"
      =dimension =thresh
    =retrieval>
      isa operator
      action test-greater-equal
      arg1 =dimension
      fail =newstep
    =game-state>
        =dimension =val     
   !safe-eval! (< =val =thresh)
     ==>
        =goal>
           prior-state nil
    +retrieval>
        isa operator
        pre =newstep)

(p delay-ok ;delay is greater than threshold
   =goal>
      state "do-step"
      time-thresh =tick-thresh
   =RETRIEVAL>
       ACTION check-delay
       success =NEWSTEP
    =temporal>
      ticks =ticks
  =game-state>
      fortress-alive yes
      VULNERABILITY =vuln
!safe-eval! (or (>= =ticks =tick-thresh)  (> =vuln 200))
     ==>
        =goal>
           prior-state nil
!safe-eval! (setf *kill* nil)
  +RETRIEVAL>
       PRE =NEWSTEP)

(p delay-not-ok  ;delay is way off-- not sure needed
   =goal>
      state "do-step"
      time-thresh =tick-thresh
   =RETRIEVAL>
       ACTION check-delay
       fail done
    =temporal>
      ticks =ticks
    ?vocal>
            state free
  =game-state>
      fortress-alive yes
!safe-eval! (> (- =tick-thresh =ticks) 10) 
     ==>
!safe-eval! (setf *kill* nil)
  +vocal>
  cmd speak
  string "wait"
    =retrieval>)

(p cant-shoot ;ship dead
   =goal>
      state "do-step"
      time-thresh =tick-thresh
   =RETRIEVAL>
       ACTION check-delay
       fail done
  =game-state>
      ship-alive no
    ?vocal>
            state free
     ==>
!safe-eval! (setf *kill* nil)
  +vocal>
  cmd speak
  string "wait"
   =goal>
      state "play")

(p tap-w ;thrust
   =goal>
      state "do-step"
      thrust-time =thrust-time
  =game-state>
      ship-alive yes
   =RETRIEVAL>
       action w
     ==>
   +MANUAL-left>
   +MANUAL>
       CMD DELAYED-PUNCH
       FINGER MIDDLE
       HAND LEFT
        delay =thrust-time
   =goal>
      state "play")

(p press-spacebar ;shoot
   =goal>
      state "do-step"
  =game-state>
      fortress-alive yes
    =retrieval>
      isa operator
      action spacebar
     ==>
    +temporal>
       isa time  
   =goal>
      state "play"
   +manual-right>
     +manual>
       isa delayed-punch
        hand right
       finger index
        delay .09)

(p double-spacebar ;kill shot
   =goal>
      state "do-step"
  =game-state>
      fortress-alive yes
    =retrieval>
      isa operator
      action double-spacebar
     ==>
   =goal>
      state "double-spacebar"
     +manual-right>
     +manual>
       isa delayed-punch
        hand right
       finger index
        delay .09)


(p double-shot ;second shot
   =goal>
      state "double-spacebar"
   ?MANUAL>
       processor FREE
     ==>
!safe-eval! (setf *kill* t)
     +manual-right>
     +manual>
       isa delayed-punch
      hand right
      finger index
       delay .09
    +temporal>
       isa time   
     =goal>
        state suspend
  +vocal>
  cmd speak
  string "wait")

(p suspend-over1 ;finished waiting to see if kill successful
   =goal>
      state suspend
      reset-bound nil
   =game-state>
       missiles nil
       ?vocal>
            state free
     ==>  
     =goal>
        state "play")

(p suspend-over2 ;finished waiting to see if kill successful when experimenting with vulnerability threshold
   =goal>
      state suspend
      reset-bound =val1
      kill-bound =val2
   =game-state>
       missiles nil
     ==>  
!bind! =newval1 (reset-bound =val1)
!bind! =newval2 (kill-bound =val2)
     =goal>
        state "play"
      reset-bound =newval1
      kill-bound =newval2)








 


(spp (attend-cause-effect SUBGOAL-AVOIDS-BORDER1 SUBGOAL-AVOIDS-BORDER2  build-shooter1 build-shooter2 double-good  GET-STARTED
      GET-STARTED-AGAIN  DETECT-TRIAL-END) :u 12 :reward t) ;piority to discovery and procedural productions
(spp (tap-w press-spacebar double-spacebar) :reward 10) ;utility reward is taking action

)


