;;;global variables for monitoring state
  (defparameter *thrust* nil) ;has a thrust been issued
  (defparameter *vulnerability* 0); current vulnerability
  (defparameter *missiles* nil); number of missiles present
  (defparameter *points* 0) ;last points
  (defparameter *data* nil) ;last data from game engine
  (defparameter *lastkey* nil) ;key identity if last update if there is a key
  (defparameter *lastaction* nil) ;last two actions and times -- e.g. (SPACEBAR 3618.666 SPACEBAR 3618.333)
   (defparameter *kill* nil) ;t when in midst of a kill shot
   (defparameter *vulnerability-kill* nil)  ;track vulnerability of recent kills
   (defparameter *vulnerability-reset* nil) ;track vulnerability of recent resetd
  (defparameter *explained* nil) ;what effects have been explained
(defparameter *prior-dif* 0) ;tracks change in inword or outward movement of ship

;parameters of model
(defparameter *detect* (random 1.0))  ;noticing probability
(defparameter *alpha* .2) ;default value for utility learning, which is reset in call
(defparameter *factor* 1.0) ;default value of starting temperature, which is reset in call
(defparameter *upper* .95) ; how close to outer constitutes touching
(defparameter *lower* .05)  ; how close to inner constitutes touching
(defparameter *time-thresh* .025) ;how fast a retrieval must be to be considered causal
(defparameter *effects* '(big-losses points-inc ship-death vuln-zero inward vuln-inc)) ;discontinuites monitored
(defparameter *factors* '(border double-spacebar spacebar double-w w)) ;possible causes
(defparameter *v* nil)

;data logging
(defparameter *changes* nil) ;data log of what effects have been associated with what factors 
;e.g. ((BIG-LOSSES 5 0 22 0 1) (POINTS-INC 5 263 0 0 0) (SHIP-DEATH 0 0 0 0 0) (VULN-ZERO 0 72 9 0 1) (INWARD 14 0 0 15 285) (VULN-INC 16 47 1116 0 43))
  (defparameter *record* nil) ;data log of causal inferences and when made
;e.g. (16.521 W INWARD 39.349 SPACEBAR VULN-INC 103.244 BORDER BIG-LOSSES 153.922 DOUBLE-SPACEBAR VULN-ZERO 634.182 DOUBLE-SPACEBAR POINTS-INC 988.766 10 VULNERABILITY)



;;; This notes relevant information for purpose of updating the imaginal and game-state module in the two games
(defun record-features-fortress (data)  
;(push data *data*)
    (update-action (car (getf data :keys)))
 (game-state-features-fortress data)
(if (equal (getf data :SCREEN-TYPE) "game")
  (let* ((vuln (getf data :vlner)) speed
         (collisions (getf data :COLLISIONS))
         (ship (getf data :ship))
         (keys (getf data :keys))
         (missiles (getf data :MISSILES))
         (points (getf data :pnts))) 
    (cond ( (and (numberp *points*) (> points *points*)) (update-slot  'kill)
          (cond ( (and (member 'points-inc *explained*) (< (random 1.0) *detect*) (<= *vulnerability* 15))
                 (push  *vulnerability* *vulnerability-kill*)))))
    (cond ((member :thrust keys) (setf *thrust* t))
          ((and *thrust* ship) (setf *thrust* nil) (setf speed (speed-sf (getf ship :vx) (getf ship :vy)))
            (if (or (< speed 1) (> speed 1.7)) (update-slot 'badspeed) (update-slot 'goodspeed))))
    (cond ((intersection collisions '(:BIG-HEX)) (update-slot 'bighexdeath))
          ((intersection collisions '(:small-HEX))  (update-slot 'smallhexdeath))
          ((intersection collisions '(:shell))  (update-slot 'shelldeath)))
    (if (intersection collisions '(:shell :BIG-HEX :small-HEX))  (update-slot 'death))
    (cond ( (and (numberp *vulnerability*)(numberp vuln) (= vuln 0) (member :FORTRESS collisions)  (< *vulnerability* 11)) (update-slot 'reset)
        (cond (*kill* (setf *kill* nil) (update-slot 'vulnerability-reset)
                      (cond  ((and (member 'points-inc *explained*) (< (random 1.0) *detect*) (>= *vulnerability* 5))
                              (push  *vulnerability* *vulnerability-reset*)))))))
    (if (and (> (length *missiles*) (length missiles)) (not (and (= vuln 0) (member :FORTRESS collisions))) (= vuln *vulnerability*)) (update-slot 'miss))
    (if (< *vulnerability* vuln)  (update-slot 'hit))
    (setf *missiles* missiles)
    (setf *points* points)
    (setf *vulnerability* vuln))))

;;; This notes relevant information for the metacognitive module after every tick
(defun game-state-features-fortress (data) 
(setf *data* data)
;(print *data*)
 (if (car (no-output (buffer-chunk game-state)))
   (cond ((equal (getf data :screen-type) "score" )
          (mod-buffer-chunk 'game-state (list 'x nil 'y nil 'vx nil 'vy nil 'fortress-alive  nil 'ship-alive nil 'VULNERABILITY nil 'outward nil
                                              'predicted-VULNERABILITY nil 
                         'prop-to-big nil  'change nil  'prior-state nil  'points nil   'missiles nil         
                    'fortress-angle nil 'orientation nil   'thrust-angle nil 'speed nil 'angle nil  'time-to-outer nil 'status 'game-over)))
         ((getf (getf data :ship) :alive)
          (let* ((ship (getf data :ship))
                 (missiles (if (getf data :missiles) t nil))
                 (fortress-alive (if (getf (getf data :fortress) :alive) 'yes 'no))
                 (old-fortress-alive (chunk-slot-value-fct (buffer-read 'game-state) 'fortress-alive))
                 (x (getf ship :x))
                 (y (getf ship :y))
                 (angle (getf ship :angle))
                 (orientation (getf ship :orientation))
                 (vel (getf ship :speed))
                 (fortress-angle (mod (- (angle x (- 630 y) 355 315) 6) 360))
                 (vdir (getf ship :vdir))
                 (vx (getf ship :vx))
                 (vy (getf ship :vy))
                 (points (getf *data* :pnts))
                 (dsmall (dist-to-hex x y :radius 40))
                 (dbig (dist-to-hex x y :radius 200))
                 (vuln (getf data :vlner))
                 (outward (outward x y vx vy))
                 (bighex (aif (getf data :bighex) it 200))
                 (game-change (if (not (equal fortress-alive old-fortress-alive)) 'state-changed 
                                (chunk-slot-value-fct (buffer-read 'game-state) 'status)))
                 (time-to-outer  (if  (> outward 0) (travel-time-to-hex vel x y vx vy :radius bighex) 10))
                 (prop-to-big (/ dsmall (+ dsmall dbig)))
                 (update (detect-change vuln points outward t))
                 (prior-state (prior-state1))
                 (old-prior-state (chunk-slot-value-fct (buffer-read 'game-state) 'prior-state))
                 (change (cond (update update)
                               ((equal old-prior-state prior-state) (chunk-slot-value-fct (buffer-read 'game-state) 'change))))
                 (predicted-vuln (if (or missiles  (query-buffer 'manual-right '(processor busy))) (1+ vuln) vuln)))
            ;(print predicted-vuln)
            (mod-buffer-chunk 'game-state  (list 'x x 'y y 'vx vx 'vy vy 'fortress-alive  fortress-alive 'ship-alive 'yes 'VULNERABILITY vuln
                     'points points 'change change 'missiles missiles 'prior-state prior-state  
                     'predicted-VULNERABILITY predicted-vuln 
                    'fortress-angle fortress-angle 'orientation orientation 'game 'spacefortress 'outward outward 'prop-to-big prop-to-big
                     'thrust-angle (- vdir angle) 'speed vel 'angle angle 'time-to-outer time-to-outer 'status game-change))))
         (t (let* 
                ((update (detect-change (getf data :vlner) (getf *data* :pnts) 0 nil))
                 (prior-state (prior-state1))
                 (old-prior-state (chunk-slot-value-fct (buffer-read 'game-state) 'prior-state))
                 (change (cond (update update)
                               ((equal old-prior-state prior-state) (chunk-slot-value-fct (buffer-read 'game-state) 'change)))))
              ;(print (list prior-state change (/ (get-time) 1000.0)))
              (mod-buffer-chunk 'game-state (list 'x nil 'y nil 'vx nil 'vy nil 'fortress-alive  (if (getf (getf data :fortress) :alive) 'yes 'no)
                                              'change change  'missiles nil 'predicted-VULNERABILITY nil
                                               'fortress-angle nil 'orientation nil 'VULNERABILITY (getf data :vlner) 'outward nil
                                               'prior-state prior-state 
                                               'prop-to-big nil  'points (getf *data* :pnts)
                                              'thrust-angle nil 'speed nil 'angle nil 'ship-alive 'no  'time-to-outer nil)))))))

;updates possible key in last game update
(defun update-action (key)
  (cond ((and key (equal key *lastkey*)) nil)
        (key (setf *lastaction* (list (classify-key key)  (/ (get-time) 1000.0) (first *lastaction*) (second *lastaction*)))  (setf *lastkey* key))
        (t (setf *lastkey* nil))))

(defun classify-key (key)
  (let* ((current-time (/ (get-time) 1000.0))
         (past-time (second *lastaction*))
         (past-action (first *lastaction*)))
    (case key
      (:fire
       (if (and (member past-action '(spacebar double-spacebar)) (< (- current-time past-time) .25)) 'double-spacebar 'spacebar))
      (:thrust
       (if (and (member past-action '(w double-w)) (< (- current-time past-time) .25)) 'double-w 'w)))))

;determines whether the new reset of kill implies a change in bounds
(defun reset-bound (val)
  (cond (*vulnerability-reset* (max (pop *vulnerability-reset*) val) )
        (t val)))

(defun kill-bound (val)
  (cond (*vulnerability-kill* (min (1- (pop *vulnerability-kill*)) val) )
        (t val)))

;determines the most recent possible cause in the last half second
(defun prior-state1 ()
  (let* ((actionTime (second *lastaction*))
         (currentTime  (/ (get-time) 1000.0))
         (action (if (and actionTime (< (- currentTime actionTime) .5)) (first *lastaction*)))
         (ship (getf *data* :ship)))
    (if ship
      (let* ( (x (getf ship :x))
             (y (getf ship :y))
             (dsmall (dist-to-hex x y :radius 40))
             (dbig  (dist-to-hex x y :radius 200))
             (prop-to-big  (/ dsmall (+ dsmall dbig))))
        (if (or (< prop-to-big *lower*) (> prop-to-big *upper*) ) 'border action)))))

;detects discontinuities
(defun detect-change (vuln points outward ship-alive)
  (if (< (random 1.0) *detect*)
    (let* ((prior-result (prior-result))
          (prior-alive (first prior-result))
          (prior-vuln (second prior-result))
          (prior-outward (third prior-result))
            (prior-points (fourth prior-result))
          (dif (if (and (numberp outward) (numberp prior-outward)) (- outward prior-outward) 0))
           (update (- dif *prior-dif*)))
      (setf *prior-dif* dif)
      (cond  ((or (not (numberp points)) (not (numberp prior-points))) nil)
             ((< (- points prior-points) -10) 'big-losses)
             ((> points prior-points) 'points-inc)
             ((< (random 1.0) .25)
              (cond ((null ship-alive) (if (eq prior-alive 'yes) 'ship-death nil))         
                    ((and prior-vuln prior-vuln (not (= prior-vuln vuln))(= vuln 0) (< (random .5) .5)) 'vuln-zero)
                    ((< (max update dif) -.2) 'inward)
                    ((and prior-vuln (not (= prior-vuln vuln)) (< (random .5) .5)) 'vuln-inc)))))))   

(defun prior-result ()
  (list (chunk-slot-value-fct (buffer-read 'game-state) 'ship-alive) 
        (chunk-slot-value-fct (buffer-read 'game-state) 'vulnerability) 
        (chunk-slot-value-fct (buffer-read 'game-state) 'outward)
        (chunk-slot-value-fct (buffer-read 'game-state) 'points)))

;computes direction of ship
(defun outward (x y vx vy)
       (let* ((x1 (- x 355))
              (y1 (- y 315))
              (dist1 (sqrt (+ (* x1 x1) (* y1 y1) )))
              (x2 (- (+ x vx) 355))
              (y2 (- (- y vy) 315))
              (dist2 (sqrt (+ (* x2 x2) (* y2 y2)))))
         (- dist2 dist1)))

;computes angle between point (x1 y1) and (x2 y2)
(defun angle (x1 y1 x2 y2)
  (let* ((run (- x2 x1))
         (rise (- y2 y1))
         (a (if (not (zerop run))  (* (atan (/ rise run)) (/ 180 pi)))))
    (cond ((zerop run) (if (>= rise 0) 90 270))
          ((> run 0) (if (>= rise 0) a (+ 360 a)))
          (t (if (>= rise 0) (+ 180 a) (+ 180 a))))))

;places effect in Imaginal buffer
(defun update-slot (slot)   
      (if (and  (car (no-output (buffer-chunk imaginal))) (not (chunk-slot-value-fct (buffer-read 'imaginal) 'effect)))
          (schedule-mod-buffer-chunk 'imaginal (list slot 1) 0)))

;speed of ship
(defun speed-sf (vx vy) (sqrt (+ (* vx vx) (* vy vy)))) 

;encodes chunks to represet discontinuities
(defun add-effects ()
    (do ((effects *effects* (cdr effects)))
        ((null effects) nil)
        (add-dm-fct (list (list (intern (concatenate 'string "PREDICT-" (string (car effects))))
                          'predicted (car effects) 'type 'prediction)))))

;produces random delays and durations of thrusts and shots when experimenting
(defun thrust-time () (+  (/ (get-time) 1000.0) 1 (random 1.0)))
(defun shoot-time () (+  (/ (get-time) 1000.0) (gam-value .6)))
(defun press-down (delay) (gam-value delay))
(defun gam-value (delay)
    (min (* delay 5.0) (max (/ delay 10.0) (- (* (log (act-r-random 1.0)) delay)))))

;tests when retrievals are fast enough to make causal inferennces
(defun retrieval-time-thresh (factor chunk) 
  (let* ((dm (get-module declarative))
         (key (hash-chunk-contents chunk))
         (existing (gethash key (bt:with-lock-held ((dm-chunk-lock dm)) (dm-chunk-hash-table dm)))) 
         (act-thresh (- (log (* 10 *time-thresh*))))
         (effect (chunk-slot-value-fct existing 'predicted)))
    (and (double-good-test factor effect) (not (member effect *explained*)) 
         (> (caar (no-output (sdp-fct (list existing :Last-Retrieval-Activation)))) act-thresh))))

(defun double-good-test (factor effect)
  (or (not (and (equal factor 'DOUBLE-SPACEBAR) (equal effect 'POINTS-INC)))
      (AND (MEMBER 'INWARD *EXPLAINED*) (or (MEMBER 'SHIP-DEATH *EXPLAINED*)  (MEMBER 'big-losses *EXPLAINED*))
           `(MEMBER 'VULN-ZERO *EXPLAINED*) (MEMBER 'VULN-ZERO *EXPLAINED*))))














;data logging and reporting
;print stats relevant to associative learning
(defun dump-associations ()
  (do ((temp *factors* (cdr temp)))
      ((null temp) 
       (princ "Total Time: ")(princ (/ (get-time) 1000)) (terpri)
       (princ "Total Associative Pairings: ") (princ  (al-module-total-count (get-module associative-learning))) (terpri)
       nil)
    (do ((temp1 *effects* (cdr temp1)))
        ((null temp1) nil)
      (princ (car temp)) (princ " ") (princ (car temp1))
      (mapcar #'(lambda (x) (princ " ") (princ x) ) (abcd (car temp) (car temp1)))
      (terpri))))

;print numbers of the 4 pairings of cause and effect present and absent
;Note the absent-absent cell is less than the total associative pairings
(defun abcd (factor effect)
      (let* (
         (effect-history (cdr (assoc effect *changes*)))
         (effect-tots (apply '+ (mapcar (lambda (x) (apply '+ (cdr x))) *changes*)))
         (position (position factor *factors*))
         (factor-history (mapcar (lambda (x) (nth (1+ position) x)) *changes*))
         (a (nth position effect-history))
         (b (- (apply '+ factor-history) a))
         (c (- (apply '+ effect-history) a))
         (d (- effect-tots a b c)))
         (list a b c d)))

;initializes *changes*
(defun blank-changes  nil 
  (do ((temp *effects* (cdr temp))
       (result nil (cons (do ((temp1 *factors* (cdr temp1))
                              (result1 nil (cons 0 result1)))
                             ((null temp1) (cons (car temp) result1))) result)))
      ((null temp) (reverse result))))

;logs when there are factor-effect pairings
(defun update-changes (prior-state change)
           (let* ((change-list (assoc change *changes*))
                  (position (1+ (position prior-state *factors*))))
             (rplaca (nthcdr position change-list) (1+ (nth position change-list)))))

;used in output of game record
(defun pad (lis n)
  (if (> (length lis) n) lis
    (do ((len (- n (length lis)) (- len 1))
         (padding nil (cons 0 padding)))
        ((= len 0) (append lis padding)))))






(defun instruction-condition (games alpha temp &optional (trace t) (speed 1) (show t))
  (run-local-model "discovery.lisp" 'run-instruction (list games (list alpha temp) trace speed show)))


;Code specific to running the instruction condition
(defun run-instruction (n params trace speed show)
   (let (points)
     (setf *alpha* (first params))
     (setf *factor* (second params))
     (setf *v* trace)
     (play-sf-games-start 1 :speed speed :condition "autoturn" :draw show)
     (setf points (list (getf *data* :total)))
     (do ((i 1 (1+ i)))
         ((= i n) (append  (pad *record* 21) points (list *detect* (chunk-slot-value-fct (buffer-read 'goal) 'predicted-vulnerability)
                                                          (chunk-slot-value-fct (buffer-read 'goal) 'thrust-angle)
                                                           (chunk-slot-value-fct (buffer-read 'goal) 'time-to-outer))))
       (play-sf-games 1 :speed speed :cont t :draw show)
       (setf points (append points (list (getf *data* :total)))))))

;inserts processing of causal instructructions before game play
(defun play-sf-games-start (games  &key (speed  1) (draw t) name condition log-format config game-number seed extra)
  (let ((module (get-module :sf)))
  (defun check-game-count (x) (declare (ignore x)) (>= (game-count module) (game-limit module)))
    (cond
      ((not module)
       (print-warning "Cannot play space fortress"))
      (t
       (reset)
       (instructions)
       (setf *explained*  '(big-losses points-inc ship-death vuln-zero inward vuln-inc VULNERABILITY))
       (goal-focus plan)
       (run 2)
       (setf (game-count module) 0)
       (setf (game-limit module) games)
       
       (when (connect-to-game module :draw draw :name name :condition condition :log-format log-format :config config :game-number game-number :seed seed :extra extra)
         (setf (game-condition module) condition)
         (schedule-event 0 'update-world :destination :sf :module :sf :priority :max :time-in-ms t)
         (run-until-condition 'check-game-count speed))))))

;causal instructions in game play including goals of getting points and avoiding losses
(defun instructions ()
  (add-dm
 (cause-effect1
   isa causal
   EFFECT  INWARD
   TYPE  CAUSAL
   cause  W)

 (cause-effect2
   isa causal
   EFFECT  VULN-INC
   TYPE  CAUSAL
   cause  SPACEBAR
   condition1 delay)

 (cause-effect3
   isa causal
   EFFECT  SHIP-DEATH
   TYPE  CAUSAL
   cause  BORDER)

 (cause-effect4
   isa causal
   EFFECT  VULN-ZERO
   TYPE  CAUSAL
   cause DOUBLE-SPACEBAR)

 (cause-effect5
   isa causal
   EFFECT  big-gain
   TYPE  CAUSAL
   cause  DOUBLE-SPACEBAR
   condition1 vuln>10
   condition2 delay)

 (cause-effect6
   isa causal
   EFFECT  BIG-LOSS
   TYPE  CAUSAL
   cause  BORDER)

(plan
   isa goal
   achieve big-gain
   avoid big-loss
   TYPE  plan
   step avoid-bad)))

;builds various operators in the instruction condition
(defun build-operator (test pre)
  (case test
    (vuln>10 (progn 
               (mod-focus predicted-vulnerability 10)
               (car (eval `(add-dm (step18
                                        ACTION  TEST-GREATER-EQUAL
                                        PRE  ,pre
                                        ARG1  predicted-VULNERABILITY
                                        FAIL  "shoot-path"
                                        SUCCESS  kill-path
                                        TYPE  OPERATOR))))))
    (delay (progn 
             (case pre
               (kill-path (car (add-dm   (step20
                                      ACTION  CHECK-DELAY
                                      PRE  kill-path
                                      FAIL  DONE
                                      SUCCESS  kill-it
                                      TYPE  OPERATOR))))
               (t (car (add-dm  (step12
                                                 ACTION  CHECK-DELAY
                                                 PRE  "shoot-path"
                                                 FAIL  DONE
                                                 SUCCESS  shoot-it
                                                 TYPE  OPERATOR)))))))
    (DOUBLE-SPACEBAR (progn 
               (car (eval `(add-dm    (step21
                                       ACTION  DOUBLE-SPACEBAR
                                       PRE  ,pre
                                       POST  DONE
                                       TYPE  OPERATOR))))))
    (w (progn 
               (car (eval `(add-dm    (step8
                                       ACTION  w
                                       PRE  ,pre
                                       success press-thrust
                                       TYPE  OPERATOR))))))
    (border (progn 
               (car (eval `(add-dm       (start
                                          ACTION  TEST-LESS
                                          PRE  ,pre
                                          ARG1  TIME-TO-OUTER
                                          FAIL  "check-shoot"
                                          success press-thrust
                                          TYPE  OPERATOR))))))
    (spacebar (progn 
               (car (eval `(add-dm      (step13
                                         ACTION  SPACEBAR
                                         PRE  shoot-it
                                         POST  DONE
                                         TYPE  OPERATOR))))))))

(defun achieve-condition (condition)
  (case condition
    (vuln>10 'VULN-INC)))

(defun oppose-condition (condition)
  (case condition
    (border 'inward)))





(defun discovery-condition (games alpha temp &optional (trace t) (speed 1) (show t))
  (run-local-model "discovery.lisp" 'run-discovery (list games (list alpha temp) trace speed show)))


;Code specific the running the discovery condition
(defun run-discovery (n params trace speed show)
   (let (points)
     (setf *alpha* (first params))
     (setf *factor* (second params))
     (setf *v* trace)
     (play-sf-games 1 :speed speed :condition "autoturn" :draw show)
     (setf points (list (getf *data* :total)))
     (do ((i 1 (1+ i)))
         ((= i n) (append  (pad *record* 21) points (list *detect* (chunk-slot-value-fct (buffer-read 'goal) 'predicted-vulnerability)
                                                          (chunk-slot-value-fct (buffer-read 'goal) 'time-thresh)
                                                           (chunk-slot-value-fct (buffer-read 'goal) 'time-to-outer))))
       (play-sf-games 1 :speed speed :cont t :draw show)
       (setf points (append points (list (getf *data* :total)))))))

;building of operator fragments in causal condition
(defun space-dm ()
  (add-dm 
    (step12
       ACTION  CHECK-DELAY
       PRE  "check-shoot"
       FAIL  DONE
       SUCCESS  shoot-it
       TYPE  OPERATOR)
    (step13
       ACTION  SPACEBAR
       PRE  shoot-it
       POST  DONE
       TYPE  OPERATOR)))

(defun w-dm ()
 (add-dm
   (step7
      ACTION  TEST-LESS
      PRE  "shoot-path"
      ARG1  TIME-TO-OUTER
      FAIL  "check-shoot"
      success press-thrust
      TYPE  OPERATOR)
   (step8
      ACTION  W
      PRE  press-thrust
      POST  DONE
      TYPE  OPERATOR)))

(defun vulnerability-dm ()
  (add-dm
   (step18
    ACTION  TEST-GREATER-EQUAL
    PRE  "start"
    ARG1  predicted-VULNERABILITY
    FAIL  "shoot-path"
    SUCCESS  kill-path
    TYPE  OPERATOR)
  (step19
     ACTION  TEST-LESS
     PRE  kill-path
     ARG1  TIME-TO-OUTER
     FAIL  kill-check
     SUCCESS  press-thrust
     TYPE  OPERATOR)
  (step20
     ACTION  CHECK-DELAY
     PRE  kill-check
     FAIL  DONE
     SUCCESS  kill-it
     TYPE  OPERATOR)
   (step21
      ACTION  DOUBLE-SPACEBAR
      PRE  kill-it
      POST  DONE
      TYPE  OPERATOR)))

;determines where to add test for shooting
(defun get-shoot-node ()
  (cond ((member 'points-inc *explained*) "start")
        ((and  (member 'inward *explained*) (or (member 'ship-death *explained*)(member 'big-losses *explained*))) "shoot-path")
        (t "check-shoot")))










